# -*- coding: utf-8 -*-
##############################################################################
#	(c)2023 by Oberhesse (contact: vuplus-support.org)
#	FREE FOR PERSONAL USE
#	CLOSED SOURCE - VTI ONLY
# 	DO NOT PUBLISH ON GITHUB
##############################################################################
	
##############################################################################
#  	---Free Code für Integration into Third Party Plugins---
#	def preLoadChannel (referenceString, interval=-1):
#		if os.path.exists("/usr/lib/enigma2/python/Plugins/Extensions/FCCExtender/FCCExtender.pyo"):
#	 		try:
#				from Plugins.Extensions.FCCExtender.FCCExtender import preLoadFCC
#				preLoadFCC( referenceString, interval) 
#	 		except: pass
#	def resetOnClose():  
#		preLoadChannel('')  #This must be done when closing Channelselsction
##############################################################################

from Components.config import config
from enigma import eServiceReference, eTimer,  eServiceCenter
from Screens.ChannelSelection import ChannelSelection, service_types_tv
from Screens.InfoBar import InfoBar
from datetime import datetime, timedelta
import os
from .__init__ import _
	
def debug(s, flag="a" , info='', fName="debug.txt"): 
	pass
	#f=open('/usr/lib/enigma2/python/Plugins/Extensions/FCCExtender/'+fName, flag);  f.write( info+str(s)+'\n'); f.close()
	
def log(s, flag="a" , info='', fName="log.txt"):  debug( s, flag, info, fName)

	
##############################################################################
	
fccTimer = None
channels = []
zapNumber = -1
lastClick = None
		
initChRef = lastChRef = InfoBarRef = timerRef = ''

def FCC_Installed():  
	return os.path.exists( "/usr/lib/enigma2/python/Plugins/SystemPlugins/FastChannelChange" )

def isActive():
	try:  return  config.plugins.FCCExtender.active.value  and  config.plugins.fccsetup.activate.value
	except:  return False

def getActiveReference():
	try:
		if InfoBar and InfoBar.instance: 
			return InfoBar.instance.servicelist.servicelist.getCurrent().toString().strip()
	except: pass
	return ''
	
def noBouquet():  #new v0.3
	try:  return InfoBar.instance.servicelist.getRoot().toCompareString().upper().find('FROM BOUQUET') < 0
	except:  return False
	
def checkRef ( sref=""):
	try:
		global lastChRef;  global initChRef;  global channels;  global InfoBarRef
		if isinstance( sref, eServiceReference): sref = sref.toString ()
		if (not sref) or (not sref in channels): return
		if not initChRef: initChRef = sref
		if noBouquet():  return
		from Plugins.SystemPlugins.FastChannelChange.plugin import FccInstance as FCC
		def refOk(ref):
			return  (FCC.isCIService(ref)==-1)  and  ref.startswith("1:0:") and FCC.isPlayableFCC(ref)
			
		if FCC and  refOk(sref) and refOk(initChRef):
			if config.plugins.fccsetup.disableforrec.value:
				FCC.getRecordings()
				if FCC.recordings:  return
				
			fccList = FCC.fccmgr.getFCCServiceList() ;
			if (not initChRef in fccList.keys()) or (InfoBarRef and not InfoBarRef in fccList.keys()):  
				return  
			if sref in fccList.keys(): return
			
			maxFCC = int(config.plugins.fccsetup.maxfcc.value)
			removed = 0
			doReplace = lastChRef and (lastChRef in fccList.keys())
			
			if len(fccList)>=maxFCC:
				for _refFCC in reversed(fccList.keys()):
					value = fccList[_refFCC]  
					state, locked = value[0], value[1]
					isIBRef = InfoBarRef and (_refFCC == InfoBarRef)
					if (not removed) and  (not state)  and (not isIBRef) and ((_refFCC==lastChRef) or not doReplace):
						try: FCC.fccmgr.stopFCCService(eServiceReference(_refFCC) ); removed = 1;  break
						except: pass
			if len(fccList) - removed < maxFCC: 
				lastChRef = sref  
				FCC.FCCTimeoutTimerStop()
				FCC.fccmgr.playFCCService(eServiceReference(sref))
				FCC.FCCTimeoutTimerStart(sref)
	except Exception as e:  debug(e,'a','checkRef')
	
	
def timerRun(*args):
	global fccTimer; global timerRef
	if fccTimer!= None: fccTimer.stop();  ref=timerRef;  timerRef=''; checkRef(ref);
	else: timerRef = ''
	

def preLoadFCC( ref, interval = -1, forceReset = 0):
	global fccTimer; global timerRef
	try:
		if interval <0: interval = int( config.plugins.FCCExtender.delay.value )
	except: 
		interval = 250
	if  (fccTimer!= None) and isActive():
		if (not ref) or forceReset:  
			reset(True)	
		else:
			fccTimer.stop();  timerRef=ref;  fccTimer.start(interval)
	else: timerRef = ''
	
##############################################################################
	
def getServiceList(ref):
	root = eServiceReference(str(ref));  serviceHandler = eServiceCenter.getInstance()
	return serviceHandler.list(root).getContent("SN", True)
	
def reset(all=0):	
	global zapNumber; global lastClick;  global lastChRef;   global initChRef;  global InfoBarRef; 
	zapNumber = -1;  lastClick = None;   InfoBarRef = ''
	if all: initChRef = lastChRef= ''
	
def new_numberEntered( self, *args, **kwargs):  
	reset(True)
	self.save_numberEntered( *args, **kwargs)

def timeout():
	try:  return config.usage.numberzap_timeout.value
	except: return 3000
	
def new_actionA( self, *args, **kwargs): 
	global zapNumber; global lastClick;  global initChRef;  global channels;  global InfoBarRef
	key, state = args[0], args[1]  
	if (key<2) or (key>11):  reset()
	elif (lastClick!=None) and  (datetime.now()>lastClick+timedelta(milliseconds=timeout()) ):  reset(True)
	res = self.save_actionA( *args, **kwargs)
	try:
		if (state==0) and (key>=2) and (key<=11):
			key = key-1   if key<11  else 0    #0..9
			if zapNumber<0: zapNumber = key
			else: zapNumber = zapNumber*10 + key
			lastClick = datetime.now() 
			if (zapNumber > 0) and (zapNumber<=len(channels)):
				initChRef =  self.getCurrentServiceRef().toString()
				InfoBarRef = getActiveReference()
				preLoadFCC( channels[zapNumber-1], 10)
	except Exception as e:  debug(e,'a','actionA')
	return res
	
		
def new_ChannelClose( self, *args, **kwargs): 
	self.save_close( *args, **kwargs)
	reset(True)

def new_updateEventInfo(self, *args, **kwargs): 
	try:
		self.save_updateEventInfo( *args, **kwargs )
		preLoadFCC  (self.getCurrentSelection().toString() )
	except Exception as e:  debug(e)
	
def new_onSelectionChanged(self, *args, **kwargs): 
	try:
		self.save_onSelectionChanged( *args, **kwargs)
		serviceRef = (self["list"].getCurrent())[1];  ref = serviceRef.ref.toString();  preLoadFCC(ref)
	except Exception as e:  debug(e,'a','osc')
	
def isMarker(ref):  #new v0.3
	try: return eServiceReference(ref).flags & (eServiceReference.isMarker|eServiceReference.isDirectory)
	except: return False
	
def init():
	def redir( cl, name, func):
		if cl and not hasattr(cl, 'save_'+name ):  setattr( cl, 'save_'+name, getattr(cl, name) );  setattr( cl, name, func )
	try:
		if FCC_Installed():
			global fccTimer;  global channels
			if fccTimer == None:   
				fccTimer=eTimer();   fccTimer.callback.append(timerRun);
			redir( ChannelSelection, 'updateEventInfo', new_updateEventInfo )
			redir( ChannelSelection, 'close', new_ChannelClose )
			redir (InfoBar, 'actionA', new_actionA)
			redir (InfoBar, 'numberEntered', new_numberEntered)
			channels, allBouquets = [], getServiceList(service_types_tv + ' FROM BOUQUET "bouquets.tv" ORDER BY bouquet') 
			for bouquet in allBouquets:
				for (ref,name) in getServiceList(bouquet[0]) :  
					if not isMarker(ref):  channels.append( str(ref) )
			if os.path.exists('/usr/lib/enigma2/python/Screens/EpgSelection.pyo'):
				from Screens.EpgSelection import EPGSelection
				redir (EPGSelection, 'onSelectionChanged', new_onSelectionChanged)
				redir( EPGSelection, 'close', new_ChannelClose )
			if os.path.exists('/usr/lib/enigma2/python/Plugins/Extensions/GraphMultiEPG/GraphMultiEpg.pyo'):
				from Plugins.Extensions.GraphMultiEPG.GraphMultiEpg import GraphMultiEPG
				redir (GraphMultiEPG, 'onSelectionChanged', new_onSelectionChanged)
				redir( GraphMultiEPG, 'close', new_ChannelClose )
	except Exception as e:  debug(e)
	
	

		